@@ -4,31 +4,32 @@ import json
 from inspect import getmembers, isclass
 import asyncio
 from enum import Enum
-from pydantic import BaseSettings
+from pydantic_settings import BaseSettings
 
 from elasticsearch import Elasticsearch
 from elasticsearch.exceptions import NotFoundError
 
-from fastapi import FastAPI, Depends, status
+from fastapi import FastAPI, Depends, status, HTTPException
 from fastapi.security import HTTPBearer, OAuth2PasswordBearer
 from fastapi.encoders import jsonable_encoder
 from fastapi.middleware.cors import CORSMiddleware
 
-from bdating_common.model import HealthResponse, ConsumerProfile, MultiLangEnum
+from bdating_common.model import HealthResponse, ConsumerProfile, MultiLangEnum, WithdrawModel
 from bdating_common import model as model_module
 # from bdating_common import model_zh as model_zh_module
 # from bdating_common import model_en as model_en_module
+from bdating_common import xero_helper
 
 from bdating_common.auth0_token_helper import IllegalTokenExcpetion, Auth0TokenVerifier
 from fastapi.responses import JSONResponse
-from bdating_common.es_helper import es_get_result_to_dict
+from bdating_common.es_helper import es_get_result_to_dict, es_search_result_to_dict
 from fastapi import WebSocket
 from fastapi.websockets import WebSocket, WebSocketDisconnect
 import redis
 import aioredis
 from aioredis.pubsub import Channel
 from auth0.v3.authentication import Users as Auth0Users
-from typing import Union, Any, Dict
+from typing import Union, Any, Dict, Optional
 
 oauth2_scheme = OAuth2PasswordBearer(tokenUrl='token')
 
@@ -89,7 +90,7 @@ class Settings(BaseSettings):
     auth0_namespace: str = 'https://app.bdating.io/'
     redis_host: str = os.getenv('REDIS_HOST')
     redis_port: int = 6379
-    redis_password: str = os.getenv('REDIS_PASSWORD')
+    redis_password: Optional[str] = os.getenv('REDIS_PASSWORD')
     cors_origins: str = os.getenv('CORS_ORIGINS', 'http://localhost:3000')
     token_cache_db_id: int = 0
     lock_db_id: int = 1
@@ -140,38 +141,6 @@ def _get_sub(token: str):
     payload_str = _get_token_payload(token)
     return json.loads(payload_str).get(f"sub")
 
-async def validate_socket_token(token: str, websocket: WebSocket):
-    global token_cache_redis
-    if settings.redis_host:
-        if token_cache_redis is None:
-            token_cache_redis = redis.Redis(
-                settings.redis_host, settings.redis_port,
-                db=settings.token_cache_db_id,
-                password=settings.redis_password
-            )
-        cached_value = token_cache_redis.get(token)
-        if cached_value is not None:
-            payload = json.loads(cached_value.decode('utf-8'))
-            return payload.get(f"{settings.auth0_namespace}wallet", payload.get('wallet'))
-    try:
-        payload = token_verifier.verify(token)
-        if payload.get("status") == 'error':
-            raise IllegalTokenExcpetion(payload)
-        wallet = payload.get(
-            f"{settings.auth0_namespace}wallet", payload.get('wallet'))
-        log.warn('wallet retrieved is %s', wallet)
-        if token_cache_redis is not None and wallet:
-            ttl = 86400 * 1000  # hard code 1 day, ttl value is in milliseconds
-            token_cache_redis.setex(
-                token, ttl,  json.dumps(payload, default=str))
-        return wallet
-    except Exception as ex:
-        log.exception(ex)
-        await websocket.send_text('{"status":"403"}')
-        await websocket.close()
-        raise IllegalTokenExcpetion(ex)
-
-
 def get_uid(token: str, token_verifier: object):
     global token_cache_redis
     if settings.redis_host:
@@ -232,12 +201,12 @@ async def ws_reader(channel: Channel, ws: WebSocket):
 
 
 async def redis_connector(
-    websocket: WebSocket, wallet: str
+    websocket: WebSocket, uid: str
 ):
     async def consumer_handler(ws: WebSocket, redis_client: object):
         try:
-            if wallet:
-                own_channel = (await redis_client.subscribe(wallet))[0]
+            if uid:
+                own_channel = (await redis_client.subscribe(uid))[0]
                 asyncio.create_task(ws_reader(own_channel, ws))
             additional_channel = None
             while True:
@@ -270,6 +239,7 @@ async def redis_connector(
     redis.close()
     await redis.wait_closed()
 
+
 async def calculate_referral_incomes(es, account_id: str):
     """
     return aggregated referral incomes data by currency
@@ -316,6 +286,7 @@ async def calculate_referral_incomes(es, account_id: str):
 async def list_referral_incomes(es, account_id: str):
     log.warning(f"listing referral incomes for account id: {account_id}")
     query = {
+        '_source':['referral_amount', 'referral_currency', 'booking.provider_uid', 'booking.consumer_uid', 'timestamp'], # Return these fields only
         "query": {
             "bool": {
                 "must": [
@@ -340,6 +311,7 @@ def create_app():
     app = FastAPI(title=settings.app_name)
     es = Elasticsearch(settings.es_endpoint)
 
+
     @ app.get("/health", response_model=HealthResponse)
     def get_health():
         return {"status": "OK"}
@@ -362,7 +334,7 @@ def create_app():
         Find this user's own profile info.
         """
         uid = get_uid(token, token_verifier)
-        return es_get_result_to_dict(es.get(index=settings.es_index, id=f"{uid}:{settings.app_type}"))
+        return es_search_result_to_dict(es.get(index=settings.es_index, id=f"{uid}:{settings.app_type}"))
 
     @app.patch("/me")
     def update_own_profile_info(profile: ConsumerProfile, token: str = Depends(token_auth_scheme)):
@@ -386,6 +358,50 @@ def create_app():
         }
         return result
 
+    @app.get('/incomes/my')
+    def get_my_incomes(
+        token: str = Depends(token_auth_scheme)
+    ):
+        uid = get_uid(token, token_verifier)
+        balance_bundle = xero_helper.get_balance_with_txns_invoices(account_id=uid)
+        txns = balance_bundle['txns']
+        invoices = balance_bundle['invoices']
+        balance = balance_bundle['balance']
+
+        booking_orders = []
+        referral_orders = []
+        for txn in txns:
+            if txn.reference.startswith('P'):
+                item = _map_xero_txn_to_resp_item(es= es, txn=txn)
+                booking_orders.append(item)
+            elif txn.reference.startswith('R'):
+                item = _map_xero_txn_to_resp_item(es= es, txn=txn)
+                referral_orders.append(item)
+
+        withdraws = list(map( lambda invoice: { 'invoice_id': invoice.invoice_id, 'amount': invoice.total, 'currency': invoice.currency_code, 'date': invoice.date, 'status': invoice.status }, invoices))
+        
+        return {
+            'aggs': {
+                'AUD': balance
+            },
+            'bookings': booking_orders,
+            'referrals': referral_orders,
+            'withdraws': withdraws
+        }
+
+    @app.post('/incomes/withdraw')
+    def request_withdraw(payload: WithdrawModel, token: str = Depends(token_auth_scheme)):
+        uid = get_uid(token, token_verifier)
+        try:
+            xero_helper.withdraw_request_received(account_id=uid, amount=payload.amount)
+            return f"Withdraw request for amount {payload.amount} {payload.currency} submitted."
+        except PermissionError as e:
+            log.info(e)
+            raise HTTPException(401, f"Error {e} when withdrawing {payload.amount} {payload.currency}.")
+        except Exception as ex:
+            log.error(ex)
+            raise HTTPException(500, f"Unexpected error when withdrawing {payload.amount} {payload.currency}.")
+
     if settings.cors_origins:
         origins = settings.cors_origins.split(',')
         app.add_middleware(
@@ -405,8 +421,8 @@ class PublishAction(Enum):
     action = 'action'
 
 
-def publish_message(message: Union[Dict[Any, Any], str], wallet: str, sync_redis: object = None):
-    """Publish a message to a user identified by wallet
+def publish_message(message: Union[Dict[Any, Any], str], uid: str, sync_redis: object = None):
+    """Publish a message to a user identified by uid
     mesage format: 
     {
         "channel": "xxxx", # what channel this message was broadcasted to. Only clients subscribed to this channel can receive this message. Yet as one client may subscribe to multiple channels, this field is kept to notify which channel this is about.
@@ -425,13 +441,12 @@ def publish_message(message: Union[Dict[Any, Any], str], wallet: str, sync_redis
     publish_message(dict(
         action=PublishAction.data.name,
         body={...}
-        ), wallet=user_id)
+        ), uid=user_id)
 
     or if simply just push a text message
-    publish_message("Please display this to the user", wallet=user_id)
+    publish_message("Please display this to the user", uid=user_id)
 
     """
-    log.info("--- api_common.py:publish_message()")
     if not sync_redis:
         sync_redis = redis.Redis(
             host=settings.redis_host,
@@ -440,13 +455,63 @@ def publish_message(message: Union[Dict[Any, Any], str], wallet: str, sync_redis
             password=settings.redis_password,
         )
     if isinstance(message, Dict):
-        sync_redis.publish(wallet, json.dumps(message, default=str))
+        sync_redis.publish(uid, json.dumps(message, default=str))
         log.info(
-            f"--- api_common.py:publish_message(): wallet={wallet}, message={json.dumps(message, default=str)}")
+            f"--- api_common.py:publish_message(): uid={uid}, message={json.dumps(message, default=str)}")
     else:
-        sync_redis.publish(wallet, json.dumps({
+        sync_redis.publish(uid, json.dumps({
             "type": "message",
             "body": {"message": message}
         }))
 
         # log.info("--- api_common.py:publish_message(): published message: " , message)
+
+def _map_xero_txn_to_resp_item(es, txn):
+    booking_fallback = {
+        'consumer_uid': 'Unknown',
+        'provider_uid': 'Unknown',
+        'date': txn.date,
+        'payment_method': txn.currency_code,
+        'payment_amount': txn.total
+    }
+    booking_id = 'UNKNOWN'
+    # xero reference, NOT referral, "P(R), booking_id=xxxxx"
+    ref = txn.reference
+    ref_arr = ref.split("booking_id=")
+    if len(ref_arr) == 2:
+        booking_id = ref_arr[-1]
+        query = {
+            '_source':['provider_uid', 'consumer_uid', 'timestamp'], # Return these fields only
+            "query": {
+                "bool": {
+                    "must": [
+                        {
+                            "match": {
+                                "type": "booking"
+                            }
+                        },
+                        {
+                            "match": {
+                                "booking_id": booking_id
+                            }
+                        }
+                    ]
+                }
+            }
+        }
+        result = es_search_result_to_dict(es.search(index=settings.es_index, body=query))
+        if len(result['results']) == 0:
+            log.warn(f"In transaction ref={ref}, booking with booking_id={booking_id} can't be found")
+            
+            return booking_fallback
+        else:
+            booking = result['results'][0]
+            # xero is the source of trust of payment currency & amount
+            booking['payment_method'] = txn.currency_code
+            booking['payment_amount'] = txn.total
+            booking['date'] = txn.date
+
+            return booking
+    else:
+        log.warn(f"In transaction ref={ref}, no booking id found")
+        return booking_fallback
