@@ -2,6 +2,7 @@ import os
 from typing import List, Optional
 from pydantic import BaseModel
 from enum import Enum
+import decimal
 
 import yaml
 
@@ -23,11 +24,10 @@ class Rating(BaseModel):
 
 
 class BaseProfile(BaseModel):
-    wallet: Optional[str]
     uid: Optional[str]
-    name: str = None
-    referrer: str = None
-    gender: str = None
+    name: str = ""
+    referrer: str = ""
+    gender: str = ""
     register_timestamp: int = 0
 
 
@@ -110,6 +110,7 @@ class SpeakingLanguage (MultiLangEnum):
 
 class PaymentMethod (MultiLangEnum):
     usdt = 'usdt'
+    card = 'card'
 
 class TimeSlotStatus(MultiLangEnum):
     available = 'available'
@@ -128,40 +129,43 @@ class BookingStatus(MultiLangEnum):
     canceled = 'canceled'
     fulfilled = 'fulfilled'
     deleted = 'deleted'  # need this? or simply delete it. as _id will conflict
+    expire_payment = 'expire_payment'
 
 
 class ProviderProfile(BaseProfile):
     address: str
-    postcode: Optional[int]
-    city: Optional[str]
-    country: Optional[str]
+    postcode: Optional[int] = None
+    city: Optional[str] = None
+    country: Optional[str] = None
     age: int = 27
     location: Location
-    contact_detail: str = None
+    contact_detail: str = ""
     rate_aud: int = 150
-    hair_color: Optional[HairColor]
-    build: Optional[Build]
-    ethnicity: Optional[Ethnicity]
-    eye_color: Optional[EyeColor]
-    bio: Optional[str]
+    hair_color: Optional[HairColor] = None
+    build: Optional[Build] = None
+    ethnicity: Optional[Ethnicity] = None
+    eye_color: Optional[EyeColor] = None
+    bio: Optional[str] = None
     photos: List[str] = []
-    height: Optional[int]
-    bust: Optional[Bust]
-    avg_on_time: Optional[float]
-    avg_service: Optional[float]
-    avg_rating: Optional[float]
-    avg_accurate_profile_description: Optional[float]
-    dress_size: Optional[DressSize]
+    height: Optional[int] = None
+    bust: Optional[Bust] = None
+    avg_on_time: Optional[float] = None
+    avg_service: Optional[float] = None
+    avg_rating: Optional[float] = None
+    avg_accurate_profile_description: Optional[float] = None
+    dress_size: Optional[DressSize] = None
     speaking_language: List[SpeakingLanguage] = []
     payment: List[PaymentMethod] = []
     instruction_images: Optional[List[str]] = []
-    instruction_text: Optional[str]
+    instruction_text: Optional[str] = None
+    email: str = None
 
 
 class ConsumerProfile(BaseProfile):
     contact: str = None
     name: str = None
     referrer: str = None
+    email: str = None
 
 class TimeSlot(ProviderProfile):
     slot_id: int  # the slot id, YYYYmmddXX
@@ -190,6 +194,7 @@ class Booking(BaseModel):  # booking to a timeslot
     provider_rating: Optional[float]
     last_update: int
     detail: BookingDetail
+    bdating_wallet: str = ''
     total_fee_aud: int
     book_time: int  # epoch second of booked
     history: List[BookingHistory] = []
@@ -261,3 +266,23 @@ class HealthResponse(BaseModel):
 
 class NotificationsModel(BaseModel):
     notifications: List[str] = []
+
+class ReferralConfig(BaseModel):
+    platform_commission: float
+    reward_layers: list[float]
+
+class RefAndCommissionModel(BaseModel):
+    uid: str
+    referral_config: ReferralConfig
+
+class InvoiceStatusEnum(str, Enum):
+    authorised = 'AUTHORISED'
+    paid = 'PAID'
+    voided = 'VOIDED'
+
+class TransactionStatusEnum(str, Enum):
+    authorised = 'AUTHORISED'
+
+class WithdrawModel(BaseModel):
+    currency: str
+    amount: decimal.Decimal
\ No newline at end of file
